Package: Rcmdr
Version: 2.12.2
Date: 2026-04-12
Title: R Commander
Encoding: UTF-8
Authors@R: c(person("John", "Fox", role = "aut"),
    person("Milan", "Bouchet-Valat", role = "aut"),
    person("Manuel", "Munoz-Marquez", role = c("aut", "cre"), email = "manuel.munoz@uca.es"),
    person("Liviu", "Andronic", role = "ctb"),
    person("Michael", "Ash", role = "ctb"),
    person("Theophilius", "Boye", role = "ctb"),
    person("Stefano", "Calza", role = "ctb"),
    person("Andy", "Chang", role = "ctb"),
    person("Vilmantas", "Gegzna", role = "ctb"),
    person("Philippe", "Grosjean", role = "ctb"),
    person("Richard", "Heiberger", role = "ctb"),
    person("Yoshinobu",  "Kanda", role = "ctb"),
    person("Kosar", "Karimi Pour", role = "ctb"),
    person("G. Jay", "Kerns", role = "ctb"),
    person("Renaud", "Lancelot", role = "ctb"),
    person("Matthieu", "Lesnoff", role = "ctb"),
    person("Uwe", "Ligges", role = "ctb"),
    person("Samir", "Messad", role = "ctb"),
    person("Martin", "Maechler", role = "ctb"),
    person("Robert", "Muenchen", role = "ctb"),
    person("Duncan", "Murdoch", role = "ctb"),
    person("Erich", "Neuwirth", role = "ctb"),
    person("Dan", "Putler", role = "ctb"),
    person("Brian", "Ripley", role = "ctb"),
    person("Miroslav", "Ristic", role = "ctb"),
    person("Peter", "Wolf", role = "ctb"),
    person("Kevin", "Wright", role="ctb")
   )
Depends: R (>= 3.5.0), grDevices, graphics, methods, stats, utils,
        splines, RcmdrMisc (>= 2.10.1), car (>= 3.1-0), effects (>=
        4.0-3)
Imports: tcltk, tcltk2 (>= 1.2-6), abind, relimp (>= 1.0-5), lme4,
        tools
Suggests: aplpack, boot, colorspace, e1071, foreign, grid, Hmisc,
        knitr, lattice, leaps, lmtest, markdown, MASS, mgcv, multcomp
        (>= 0.991-2), nlme, nnet, nortest, readxl, rgl (>= 0.110.2),
        rmarkdown (>= 0.9.5), sem (>= 2.1-1)
ByteCompile: yes
Description: 
  A platform-independent basic-statistics GUI (graphical user interface) for R, based on the tcltk package.
License: GPL (>= 2)
URL: https://github.com/RCmdr-Project/rcmdr, https://www.r-project.org,
        https://www.john-fox.ca/RCommander/index.html
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-10 19:57:13 UTC; mmarquez
Author: John Fox [aut],
  Milan Bouchet-Valat [aut],
  Manuel Munoz-Marquez [aut, cre],
  Liviu Andronic [ctb],
  Michael Ash [ctb],
  Theophilius Boye [ctb],
  Stefano Calza [ctb],
  Andy Chang [ctb],
  Vilmantas Gegzna [ctb],
  Philippe Grosjean [ctb],
  Richard Heiberger [ctb],
  Yoshinobu Kanda [ctb],
  Kosar Karimi Pour [ctb],
  G. Jay Kerns [ctb],
  Renaud Lancelot [ctb],
  Matthieu Lesnoff [ctb],
  Uwe Ligges [ctb],
  Samir Messad [ctb],
  Martin Maechler [ctb],
  Robert Muenchen [ctb],
  Duncan Murdoch [ctb],
  Erich Neuwirth [ctb],
  Dan Putler [ctb],
  Brian Ripley [ctb],
  Miroslav Ristic [ctb],
  Peter Wolf [ctb],
  Kevin Wright [ctb]
Maintainer: Manuel Munoz-Marquez <manuel.munoz@uca.es>
Repository: CRAN
Date/Publication: 2026-04-11 06:21:29 UTC
