# Generated by roxygen2: do not edit by hand

S3method(Anova,glmerMod)
S3method(Anova,lmerMod)
S3method(Coef,default)
S3method(Coef,merMod)
S3method(Coef,multinom)
S3method(anova,glmerMod)
S3method(anova,lmerMod)
S3method(coef,glmerMod)
S3method(coef,lmerMod)
S3method(editDataset,"NULL")
S3method(editDataset,character)
S3method(editDataset,data.frame)
S3method(getFrame,combobox)
S3method(getFrame,listbox)
S3method(getSelection,combobox)
S3method(getSelection,listbox)
S3method(linearHypothesis,glmerMod)
S3method(linearHypothesis,lmerMod)
S3method(plot,glmerMod)
S3method(plot,lmerMod)
export("tclvalue<-")
export(.Tcl)
export(ActiveDataSet)
export(ActiveModel)
export(Character)
export(Coef)
export(Coefficients)
export(CoefficientsP)
export(Commander)
export(CommanderWindow)
export(DiscreteNumeric)
export(EffectP)
export(Factors)
export(GrabFocus)
export(Library)
export(LogWindow)
export(MacOSXP)
export(MarkdownP)
export(Message)
export(MessagesWindow)
export(Numeric)
export(NumericPositive)
export(OKCancelHelp)
export(OutputWindow)
export(Predictors)
export(PredictorsP)
export(RExcelSupported)
export(RappP)
export(RcmdrPager)
export(RcmdrTclSet)
export(RcmdrTkmessageBox)
export(RecodeDialog)
export(RmdWindow)
export(RnwWindow)
export(TwoLevelFactors)
export(UpdateModelNumber)
export(Variables)
export(WindowsP)
export(X11P)
export(activateMenus)
export(activeDataSet)
export(activeDataSetP)
export(activeModel)
export(activeModelP)
export(anovaP)
export(beginRmdBlock)
export(beginRnwBlock)
export(buttonRcmdr)
export(characterP)
export(checkActiveDataSet)
export(checkActiveModel)
export(checkBoxes)
export(checkClass)
export(checkFactors)
export(checkMethod)
export(checkNumeric)
export(checkReplace)
export(checkTwoLevelFactors)
export(checkVariables)
export(closeCommander)
export(closeDialog)
export(commanderPosition)
export(dataSetsP)
export(defmacro)
export(dialogSuffix)
export(discreteNumericP)
export(doItAndPrint)
export(editDataset)
export(endRmdBlock)
export(endRnwBlock)
export(enterKnitr)
export(enterMarkdown)
export(errorCondition)
export(exists.method)
export(factorsP)
export(flushDialogMemory)
export(formulaFields)
export(gassign)
export(getCases)
export(getDialog)
export(getFrame)
export(getRcmdr)
export(getSelection)
export(gettextRcmdr)
export(glmP)
export(groupsBox)
export(groupsLabel)
export(hclustSolutionsP)
export(hierarchicalCluster)
export(initializeDialog)
export(insertRmdSection)
export(is.SciViews)
export(is.valid.name)
export(is.valid.number)
export(justDoIt)
export(knitrP)
export(labelRcmdr)
export(linearModel)
export(listAOVModels)
export(listAllModels)
export(listCharacter)
export(listDataSets)
export(listDiscreteNumeric)
export(listFactors)
export(listGeneralizedLinearModels)
export(listLinearModels)
export(listMultinomialLogitModels)
export(listNumeric)
export(listNumericPositive)
export(listPlugins)
export(listProportionalOddsModels)
export(listTwoLevelFactors)
export(listVariables)
export(lmP)
export(logLikP)
export(logger)
export(manualTranslationP)
export(mavericksP)
export(modelCapability)
export(modelFormula)
export(modelsP)
export(multinomP)
export(numericP)
export(numericPositiveP)
export(packageAvailable)
export(polrP)
export(popCommand)
export(popOutput)
export(putDialog)
export(putRcmdr)
export(radioButtons)
export(removeLastRmdBlock)
export(removeLastRnwBlock)
export(removeNullRmdBlocks)
export(removeNullRnwBlocks)
export(removeStrayRmdBlocks)
export(removeStrayRnwBlocks)
export(rglLoaded)
export(saveOptions)
export(setBusyCursor)
export(setIdleCursor)
export(showData)
export(sortVarNames)
export(subOKCancelHelp)
export(subsetBox)
export(suppressMarkdown)
export(tcl)
export(tclRequire)
export(tclServiceMode)
export(tclVar)
export(tclvalue)
export(titleLabel)
export(tk_select.list)
export(tkbind)
export(tkcget)
export(tkcheckbutton)
export(tkchooseDirectory)
export(tkconfigure)
export(tkcurselection)
export(tkdelete)
export(tkdestroy)
export(tkentry)
export(tkfocus)
export(tkframe)
export(tkgetOpenFile)
export(tkgetSaveFile)
export(tkgrab.release)
export(tkgrab.set)
export(tkgrid)
export(tkgrid.columnconfigure)
export(tkgrid.configure)
export(tkgrid.rowconfigure)
export(tkicursor)
export(tkindex)
export(tkinsert)
export(tkinvoke)
export(tkitemconfigure)
export(tklabel)
export(tklistbox)
export(tkmark.set)
export(tkmessageBox)
export(tkpack)
export(tkradiobutton)
export(tkraise)
export(tkscale)
export(tkscrollbar)
export(tkselection.clear)
export(tkselection.present)
export(tkselection.set)
export(tkset)
export(tkspinbox)
export(tktag.configure)
export(tktext)
export(tktoplevel)
export(tkwait.window)
export(tkwm.deiconify)
export(tkwm.geometry)
export(tkwm.iconbitmap)
export(tkwm.protocol)
export(tkwm.resizable)
export(tkwm.title)
export(tkwm.transient)
export(tkxview)
export(tkxview.moveto)
export(tkyview)
export(tkyview.scroll)
export(trim.blanks)
export(ttkcheckbutton)
export(ttkentry)
export(ttkframe)
export(ttklabelframe)
export(ttkradiobutton)
export(ttkscrollbar)
export(twoLevelFactorsP)
export(varPosn)
export(variableComboBox)
export(variableListBox)
import(RcmdrMisc)
import(car)
import(effects)
import(splines)
import(tcltk)
importFrom(abind,abind)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorConverter)
importFrom(grDevices,colorspaces)
importFrom(grDevices,convertColor)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.new)
importFrom(grDevices,dev.size)
importFrom(grDevices,palette)
importFrom(grDevices,rgb)
importFrom(graphics,text)
importFrom(lme4,fixef)
importFrom(methods,as)
importFrom(methods,getClass)
importFrom(methods,isVirtualClass)
importFrom(methods,show)
importFrom(relimp,showData)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,family)
importFrom(stats,formula)
importFrom(stats,getCall)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,qnorm)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(tcltk,"tclvalue<-")
importFrom(tcltk,.Tcl)
importFrom(tcltk,tcl)
importFrom(tcltk,tclRequire)
importFrom(tcltk,tclServiceMode)
importFrom(tcltk,tclVar)
importFrom(tcltk,tk_select.list)
importFrom(tcltk,tkbind)
importFrom(tcltk,tkcget)
importFrom(tcltk,tkcheckbutton)
importFrom(tcltk,tkchooseDirectory)
importFrom(tcltk,tkconfigure)
importFrom(tcltk,tkcurselection)
importFrom(tcltk,tkdelete)
importFrom(tcltk,tkdestroy)
importFrom(tcltk,tkentry)
importFrom(tcltk,tkfocus)
importFrom(tcltk,tkframe)
importFrom(tcltk,tkgetOpenFile)
importFrom(tcltk,tkgetSaveFile)
importFrom(tcltk,tkgrab.release)
importFrom(tcltk,tkgrab.set)
importFrom(tcltk,tkgrid)
importFrom(tcltk,tkgrid.columnconfigure)
importFrom(tcltk,tkgrid.configure)
importFrom(tcltk,tkgrid.rowconfigure)
importFrom(tcltk,tkicursor)
importFrom(tcltk,tkindex)
importFrom(tcltk,tkinsert)
importFrom(tcltk,tkinvoke)
importFrom(tcltk,tkitemconfigure)
importFrom(tcltk,tklabel)
importFrom(tcltk,tklistbox)
importFrom(tcltk,tkmark.set)
importFrom(tcltk,tkmessageBox)
importFrom(tcltk,tkpack)
importFrom(tcltk,tkradiobutton)
importFrom(tcltk,tkraise)
importFrom(tcltk,tkscale)
importFrom(tcltk,tkscrollbar)
importFrom(tcltk,tkselection.clear)
importFrom(tcltk,tkselection.present)
importFrom(tcltk,tkselection.set)
importFrom(tcltk,tkset)
importFrom(tcltk,tktag.configure)
importFrom(tcltk,tktext)
importFrom(tcltk,tktoplevel)
importFrom(tcltk,tkwait.window)
importFrom(tcltk,tkwm.deiconify)
importFrom(tcltk,tkwm.geometry)
importFrom(tcltk,tkwm.iconbitmap)
importFrom(tcltk,tkwm.protocol)
importFrom(tcltk,tkwm.resizable)
importFrom(tcltk,tkwm.title)
importFrom(tcltk,tkwm.transient)
importFrom(tcltk,tkxview)
importFrom(tcltk,tkxview.moveto)
importFrom(tcltk,tkyview)
importFrom(tcltk,tkyview.scroll)
importFrom(tcltk,ttkcheckbutton)
importFrom(tcltk,ttklabelframe)
importFrom(tcltk2,is.ttk)
importFrom(tcltk2,tk2style)
importFrom(tcltk2,tk2table)
importFrom(tcltk2,tk2theme)
importFrom(tcltk2,tk2theme.list)
importFrom(tcltk2,tk2tip)
importFrom(utils,assignInMyNamespace)
importFrom(utils,available.packages)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,help)
importFrom(utils,help.start)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,methods)
importFrom(utils,read.table)
importFrom(utils,tail)
