#' @name reexports
#' 
#' @title \pkg{tcltk} Functions Re-exported by the \pkg{Rcmdr} Package.
#' 
#' @keywords misc
#' 
#' @author John Fox
#'
#' @details
#' For technical reasons involving \pkg{Rcmdr} plug-in packages, the \pkg{Rcmdr} package re-exports a number of functions from the \pkg{tcltk} package that are used by \pkg{Rcmdr} utilities.
#'
#' It also re-exports the \code{showdata} function from the \pkg{relimp} package, which is uses internally to display data sets.
#'
#' @seealso \code{\link{tcltk}}.
#'
NULL

