globalVariables(c('languageVariable', 'top', 'statisticFrame', 'statisticVariable', 'buttonsFrame',
                  '.activeModel', 'varVariable', 'rhsVariable', 'varFrame', 'formulaFrame',
                  'outerOperatorsFrame', 'xBox', 'matrixVariable', 'factorCorVariable',
                  'identifyVariable', 'robustVariable', 'matrixFrame', 'factorCorFrame',
                  'identifyFrame', 'idtext', 'robustFrame', 'regexprVariable', 'caseVariable',
                  'directionVariable', 'optionsFrame', 'directionFrame', 'putRExcel',
                  'altHypothesisVariable', 'altHypothesisFrame', '.Responses',
                  'scaleVariable', 'scaleFrame', 'centerVariable', 'centerFrame', 'console.output',
                  'OKbutton', 'typeFrame', 'messageBox', 'groupsFrame', 'current.print.line',
                  'Xyplot.HH', 'auto.keyVariable', 'outerVariable', 'x.relationVariable',
                  'y.relationVariable', 'x.relationFrame', 'y.relationFrame', 'fittedVariable',
                  'residualsVariable', 'rstudentVariable', 'hatvaluesVariable', 'cookdVariable',
                  'obsNumbersVariable', 'selectFrame', 'typeVariable', 'levelsVariable',
                  'methodVariable', 'subdialog', 'subButtonsFrame', 'entry1', 'onCancel',
                  'levelNames', 'levelsFrame', 'methodFrame', 'correlationsVariable',
                  'correlationsFrame', 'alternativeVariable', 'alternativeFrame', '.x', '.Table',
                  'meanVariable', 'sumVariable', 'sdVariable', 'checkBoxFrame', 'percentsVariable',
                  'chisqVariable', 'chisqComponentsVariable', 'expFreqVariable', 'fisherVariable',
                  '.Test', 'percentsFrame', 'testsFrame', 'delimiterFrame', 'delimiterVariable',
                  'colnamesVariable', 'rownamesVariable', 'quotesVariable', 'subsetVariable', 'subsetEntry',
                  'rotationVariable', 'scoresVariable', '.FA', 'subsetFrame', 'rotationFrame',
                  'scoresFrame', 'subwin', '.Probs', 'lhsVariable', 'onHelp', 'lhsEntry',
                  'distanceTypeVariable', 'distanceTypeFrame', 'queryimportRODBCtable',
                  'variancesVariable', '.groupsLabel', 'variancesFrame', 'helpButton',
                  'numbersButton', 'namesButton', 'cvVariable', 'skewnessVariable',
                  'kurtosisVariable', 'typeButtonsVariable', '.groups', 'skCheckBoxFrame',
                  'typeButtonsFrame', '.Pairs', 'proportionalOddsModel', 'modelTypeVariable',
                  'modelTypeFrame', 'testVariable', 'testFrame', 'screeplotVariable',
                  'addPCVariable', '.PC', 'locationVariable', 'decimalVariable', 'locationFrame',
                  'decimalFrame', 'TempEnv', 'filetypeVariable', 'filetypeFrame', 'jitterXVariable',
                  'jitterYVariable', 'logXVariable', 'logYVariable', 'boxplotsVariable',
                  'lsLineVariable', 'smoothLineVariable', 'spreadVariable', 'diagonalVariable',
                  'diagonalFrame', 'contrastsVariable', 'contrastsFrame', '..', 'hex.1', 'hex.2',
                  'hex.3', 'hex.4', 'hex.5', 'hex.6', 'hex.7', 'hex.8', '.Z', 'partsVariable',
                  'styleVariable', 'trimOutliersVariable', 'showDepthsVariable',
                  'reverseNegativeVariable', 'partsFrame', 'styleFrame', 'criterionVariable',
                  'criterionFrame', 'plotTypeVariable', 'plotTypeFrame', '.RHS',
                  'chisqTestVariable', 'fisherTestVariable', '.commander.done', '.Workbook', '.Datasets', 
                  'getSheets', 'odbcConnectExcel', 'odbcConnectExcel2007', 'odbcConnectAccess', 
                  'odbcConnectAccess2007', 'odbcConnectDbase', 'odbcCloseAll', 'sqlTables', 'sqlQuery',
                  'IQRVariable',  'unitsVariable', 'unitsFrame',
                  'pointsVariable', 'linesVariable', 'useVariable', 'useFrame',
                  'askToExitVariable', 'askOnExitVariable', 'quitRVariable', 'consoleOutputVariable',
                  'numberMessagesVariable', 'retainMessagesVariable', 'useMarkdownVariable', 'useKnitrVariable', 'logCommandsVariable',
                  'grabFocusVariable', 'doubleClickVariable', 'sortNamesVariable', 'showEditButtonVariable',
                  'SuppressIconImagesVariable', 'retainSelectionsVariable', 'useRglVariable', 'closeOptionsFrame',
                  'outputOptionsFrame', 'otherOptionsFrame', 'kernelVariable', 'kernelFrame', 'otherFrame',
                  'notebook', 'dataTab', 'optionsTab', 'tableTab', 'statisticsTab', 'allEffectsVariable', 'allEffectsFrame', 
                  'styleButtonsVariable', 'styleButtonsFrame', 'se.meanVariable',
                  'AIC', 'BIC', 'GFI', 'AGFI', 'RMSEA', 'NFI', 'NNFI', 'CFI', 'RNI', 'IFI', 'SRMR', 'AICc', 'CAIC',
                  'fitIndicesFrame.1', 'fitIndicesFrame.2', 'sandwichTypeVariable', 'sandwichTypeFrame',
                  'rgl.cur', 'rgl.bringtotop', '.Contrasts', 'appnapVariable', 'appnapFrame',
                  'formatButtonsVariable', 'formatButtonsFrame', 'latexVariable', 'pandocVariable', 'selectSoftwareFrame',
                  'legendVariable', 'legendFrame', 'legendPositionVariable', 'legendPositionFrame',
                  'valuesOrQuantilesVariable', 'valuesOrQuantilesFrame',
                  'conditionalVariable', 'colorsVariable', 'colorsFrame', 'conditionalFrame',
                  'countsVariable', 'allPredictorEffectsFrame', 'allPredictorEffectsVariable', 'familyFrame', 'familyVariable',
                  'bonfVariable', 'cookVariable', 'hatVariable', 'studentVariable', 'varsFrame', 'estimTypeFrame', 'estimTypeVariable',
                  '.responseLabel', 'testStatisticVariable', 'designTab', 'testStatisticFrame', 'oneFactorTab', 'twoFactorsTab',
                  'modelCaseDeletionVariable'))
