\documentclass{article}%
\usepackage[round]{natbib}   % For in-text citations
\bibliographystyle{apalike} % For bibliography style
\usepackage{amsmath}%
\setcounter{MaxMatrixCols}{30}%
\usepackage{amsfonts}%
\usepackage{amssymb}%
\usepackage{graphicx}
%\usepackage{url}
\usepackage{hyperref} 

\usepackage[spanish]{babel}
\usepackage[utf8]{inputenc}

\setlength{\oddsidemargin}{0in}
\setlength{\evensidemargin}{0in}
\setlength{\topmargin}{-0.5in}
\setlength{\textwidth}{6.5in}
\setlength{\textheight}{9in}

%\VignetteIndexEntry{Iniciación a R Commander}
%\VignetteEncoding{UTF-8}

\begin{document}

<<echo = FALSE, results = hide>>=
Sys.setenv(LANGUAGE='es')
Sys.setLanguage("es")
file.copy(from = 'Rcmdr-menus', to = 'Rcmdr-menus-es', overwrite = TRUE)
Sweave('Rcmdr-menus-es')
@ 

\title{Iniciación a R Commander\thanks{Parte de este manual ha sido adaptado y actualizado de Fox (2005). 
Por favor, dirija la correspondencia al mantendor actual Manuel Munoz-Marquez \href{mailto:manuel.munoz@uca.es}{\texttt{manuel.munoz@uca.es}}}\\ (Getting Started With the R Commander)}

\author{John Fox\\ Traductor: Manuel Munoz-Marquez\thanks{N.T.: Para la traducción al español diríjase a \href{mailto:manuel.munoz@uca.es}{\texttt{manuel.munoz@uca.es}}}}

\date{Versión 2.6-2 (última modificación: 2023-04-28)}

\maketitle

\section{Introducción}

\textbf{R Commander} \citep{Fox05, Fox17} proporciona una interfaz gráfica de usuario o \textquotedblleft GUI\textquotedblright\footnote{N.T.: Del inglés Graphical User Interface} al entorno estadístico abierto \textbf{R} \citep{RCore20}. 
Este manual es una introducción breve y básica a \textbf{R Commander}, para una documentación más extensa véase~\citet{Fox17}, que tiene un sitio web en \linebreak <\href{https://www.john-fox.ca/RCommander/index.html}{www.john-fox.ca/RCommander/index.html}.

\textbf{R} es un sistema orientado a instrucciones y los nuevos usuarios a menudo encuentran el aprendizaje de \textbf{R} difícil. 
Esto es particularmente cierto para aquellos que son nuevos en métodos estadísticos, como los estudiantes en los cursos de estadística básica. 
Mediante la provisión de un interfaz gráfico para \textbf{R}, \textbf{R Commander}, se permite a estos usuarios que se concentren en los métodos estadísticos en vez de en recordar y construir instrucciones para \textbf{R}.  
Además, al construir las instrucciones de forma visible a los usuarios, \textbf{R Commander} permite una fácil transición a la escritura de instrucciones para \textbf{R}, al menos para algunos usuarios. 
Sin embargo, \textbf{R Commander} proporciona acceso sólo a una pequeña parte de las prestaciones de \textbf{R} y de los miles de paquetes para \textbf{R} proporcionados por los usuarios existentes en ``Comprehensive R Archive
Network (CRAN)". 
\textbf{R Commander} es extensible mediante la creación de paquetes de añadidos y muchos de dichos paquetes de extensiones están disponibles en CRAN (véase la sección~\ref{sec-plugins} de este documento).

Este documento describe el uso de \textbf{R Commander} bajo \textbf{R} para \textbf{Microsoft Windows}. 
Hay algunas pequeñas diferencias en la apariencia y el uso de \textbf{R Commander} bajo \textbf{Mac OS X} y en sistemas \textbf{Linux} y \textbf{Unix}. 
En el enlace a las notas de instalación en la página de \textbf{R Commander} \href{https://www.john-fox.ca/RCommander/installation-notes.html}{www.john-fox.ca/RCommander/installation-notes.html} hay información disponible sobre la instalación de \textbf{R Commander} en estas plataformas 

Usaremos las siguientes convenciones tipográficas en este documento: los nombres de software, como \textbf{Windows}, \textbf{R}, el paquete \textbf{Rcmdr} y \textbf{R Commander}, se escriben en \textbf{negrita}. 
Los nombre de elementos de la interfaz gráfica como menús, elementos de menú, ventanas y cuadros de diálogo, se escriben en \emph{itálica}. 
Los nombres de variables, nombres de conjuntos de datos e instrucciones de \textbf{R} van en \texttt{fuente tipo máquina de escribir}.

\section{Comenzando con R Commander}

Con \textbf{R} ejecutándose, cargando el paquete \textbf{Rcmdr} tecleando la instrucción \texttt{library(Rcmdr)} en la consola \emph{R Console} se inicia la interfaz gráfica \textbf{R Commander}. 
El funcionamiento óptimo bajo \textbf{Windows}, de \textbf{R Commander} es usando la interfaz de documento único (\textquotedblleft SDI\textquotedblright) para \textbf{R}\footnote{La versión de \textbf{R} para \textbf{Windows} corre normalmente en la interfaz de documento múltiple (\textquotedblleft MDI\textquotedblright), el cual contiene la ventana de la consola \emph{R Console}, la ventana de gráficos \emph{Graphical Device} creada durante la sesión y otras ventanas relacionadas con el proceso de \textbf{R}. 
En contraposición, baja la interfaz de documento único (\textquotedblleft
SDI\textquotedblright), la consola \emph{R Console} y la ventana gráfica \emph{Graphical Device} no están contenidas en la ventana principal. 
Hay muchas formas de ejecutar \textbf{R} en modo SDI --- por ejemplo, seleccionando SDI cuando se instala \textbf{R}, o editando el fichero \texttt{Rconsole} en el subdirectorio \texttt{etc} de \textbf{R}, o añadiendo \texttt{-{}-sdi} al campo \emph{Destino} en las propiedades del icono de \textbf{R} en el escritorio. 
Debería de ser capaz de usar \textbf{R Commander} bajo MDI, pero no aparecerá dentro de la ventana principal de \textbf{R} y la distribución de las ventanas no será la apropiada.}. 
Depués de cargar el paquete, las ventanas \emph{R Console} y \emph{R Commander} deberían aparecer más o menos como en las figuras \ref{fig-1} y \ref{fig-1-b}\footnote{Muchas de las capturas de pantalla de este documento se realizaron con versiones anteriores de \textbf{R} y de \textbf{R Commander}. 
Las imágenes de las pantallas son actualizadas únicamente cuando cambia su apariencia o su contenido.}. 
Estas y otras imágenes de pantallas en este documento se han realizado bajo \textbf{Windows 10}; si usa otra versión de \textbf{Windows} (o, por supuesto, otro sistema operativo), entonces la apariencia de las pantallas puede cambiar\footnote{Observe que \textbf{R Commander} necesita algunos paquetes además de los muchos paquetes \textquotedblleft recomendados \textquotedblright\ que son normalmente distribuidos con \textbf{R}. 
El paquete \textbf{Rcmdr}, sus paquetes requeridos y muchos otros paquetes de contribuciones están disponibles para descarga en ``Comprehensive R Archive Network" (CRAN) en \href{https://cran.r-project.org/}{https://cran.r-project.org/}. 

Si estos paquetes no se instalan, \textbf{R Commander} pedirá su instalación desde internet o desde ficheros locales (por ejemplo, desde un CD-ROM desde una memoria USB). 
Si instala el paquete \textbf{Rcmdr} en \textbf{Windows} mediante \textquotedblleft
R GUI,\textquotedblright\ no todos los paquetes de los que depende \textbf{Rcmdr} serán instalados. 
Puede instalar el paquete \textbf{Rcmdr} y todos los paquetes de los que depende mediante la función \texttt{install.packages}, usando la opción \texttt{dependencies = TRUE}, pero debido a las dependencias recursivas, esto instalará más paquetes de los estrictamente necesarios para el funcionamiento de \textbf{R Commander}. 

Gracias a Dirk Eddelbuettel, los usuarios de \textbf{Debian Linux} únicamente necesitan la instrucción \texttt{\$ apt-get install r-cran-rcmdr} para instalar \textbf{Rcmdr} junto a todos los paquetes que necesita. 
En cualquier caso, la construcción e instalación del paquete \textbf{Rcmdr} en sistemas \textbf{Linux} es habitualmente directa. 
La tarea es un poco más complicada bajo \textbf{Mac OS X}, debido a que el paquete \textbf{tcltk} del que depende 
\textbf{Rcmdr} requiere tener \textbf{X-Windows} instalado --- véase \textbf{R Commander} installation notes.}~\footnote{N.T.: Las capturas en la versión española se han realizado bajo ubuntu-linux.}.

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=6.560400in,
%natwidth=9.080100in,
%height=3.4388in,
%width=4.7497in
scale=.4
]%
{fig1-es}%
\caption{La ventana \emph{R Console} después de cargar el paquete \textbf{Rcmdr}.}%
\label{fig-1}%
\end{figure}

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=10.800300in,
%natwidth=9.293400in,
%height=5.6272in,
%width=4.846in
scale=.4
]%
{fig-1-b-es}%
\caption{\emph{R Commander} al inicio.}%
\label{fig-1-b}%
\end{figure}

Las ventanas \emph{R Commander} y \emph{R Console} se mueven libremente en el escritorio. 
Normalmente usará los menús y las cajas de diálogo de \textbf{R Commander} para leer, manipular y analizar datos y puede minimizar sin riesgo la ventana \emph{R Console}.

\begin{itemize}
\item Las instrucciones de \textbf{R} generadas por \textbf{R Commander} aparecerán en el cuadro \emph{R Script} en la parte superior de la ventana principal de \emph{R Commander}. 
También puede teclear instrucciones de \textbf{R} directamente en el cuadro \emph{R Script}\footnote{También puede teclear instrucciones en el símbolo \texttt{$>$} (mayor que) de \emph{R Console}, pero la salida generada por estas instrucciones no aparecerá en el cuadro \emph{Salida} de \textbf{R Commander} y los errores y avisos no serán normalmente visibles.}; 
el objetivo principal de \textbf{R Commander}, no obstante, es evitar la necesidad de teclear instrucciones. 
La segunda pestaña en la parte superior (etiquetada \emph{R Markdown}) también va conteniendo las instrucciones producidas por \textbf{R Commander} y puede ser usada para generar informes para imprimir; la pestaña \emph{R Markdown} se describe en la sección~\ref{sec-markdown}.

\item La salida impresa aparece por defecto en el segundo cuadro (etiquetado \emph{Output}).

\item El cuadro inferior gris, (etiquetado \emph{Mensajes}) muestra los mensajes de error, avisos y alguna otra información (\textquotedblleft Notas\textquotedblright), como el mensaje de inicio en la figura~\ref{fig-1-b}.

\item Cuando cree un gráfico, este aparecerá en una ventana separada \emph{Graphics Device}.
\end{itemize}

Hay muchos menús en la parte alta de la ventana de \emph{R Commander}:

\begin{description}
\item[Fichero] Entradas de menú para cargar y salvar ficheros de instrucciones, para guardar la salida y los espacios de trabajo de \textbf{R} y para salir.

\item[Editar] Entradas de menú (\emph{Cortar}, \emph{Copiar}, \emph{Pegar}, etc.) para editar texto en varios cuadros y pestañas. 
Pulsando el botón derecho sobre uno de estos cuadros o pestañas también se despliega un menú de edición \textquotedblleft contextual\textquotedblright.

\item[Datos] Submenú que contiene entradas de menú para la lectura y manipulación de datos.

\item[Estadísticos] Submenús que contienen opciones para una gran variedad de análisis estadísticos.

\item[Gráficas] Entradas de menú para la creación de varios gráficos estadísticos.

\item[Modelos] Entradas de menú y submenús para la obtención de resúmenes numéricos, intervalos de confianza, tests de hipótesis, diagnósticos y gráficos para modelos estadísticos y para añadir diagnósticos cuantitativos, como residuos, a los conjuntos de datos. 

\item[Distribuciones] Submenús para obtener las probabilidades acumuladas, densidades o masas de probabilidad, cuantiles y gráficos de distribuciones estándares (para ser usadas, por ejemplo, como un sustituto de tablas estadísticas y para generar muestras de esas distribuciones.

\item[Herramientas] Entradas de menú para la carga de paquetes de \textbf{R} no relacionados con el paquete \textbf{Rcmdr} (por ejemplo, para acceder a datos guardados en otro paquete); para cargar paquetes de extensiones de \textbf{Rcmdr} (véase~\citealp{Fox07},~\citealp{FoxCarvalho12} y la sección~\ref{sec-plugins} más adelante);
para establecer la mayoría de las opciones de \textbf{\ R Commander} y para guardarlas de manera que se apliquen en las siguientes sesiones y para la instalación de software auxiliar opcional (véase la sección~\ref{sec-markdown}).

\item[Ayuda] Entradas de menú para obtener información sobre \textbf{R Commander} (incluyendo este manual\footnote{N.T.: Están disponibles en este menú tanto la versión traducida al español como la versión inglesa. Habitualmente la versión inglesa es más reciente.}) y el software asociado. 
Además, cada cuadro de diálogo de \textbf{R Commander} tiene un botón de \emph{Ayuda} (véase más adelante).
\end{description}

El \textquotedblleft árbol\textquotedblright\ completo del menú de \textbf{R Commander} (versión 2.8-0) se muestra abajo. 
La mayoría de las entradas de menú llevan a un cuadro de diálogo, como se ilustra más tarde en este manual y, como es convencional, dichas entradas de menú van seguidas de \texttt{...}. 
Las entradas de menú están inactivas (\textquotedblleft sombreadas \textquotedblright) si no son aplicables en el contexto actual. 
Por ejemplo, si el conjunto de datos no contiene factores (variables categóricas), la entrada de menú para las tablas de contingencia estará inactiva\footnote{Algunas opciones de menú no son mostradas en algunas circunstancias.
Por ejemplo, las entradas de menú de \textbf{knitr} en el menú \emph{Fichero} se mostrarán únicamente si la pestaña \emph{knitr} es activada y el elemento \emph{Instalar software auxiliar} en el menú \emph{Herramientas} se activa solo si el software opcional \LaTeX{} o \textbf{Pandoc} no están instalados.
Los menús también incluyen divisores, que no son mostrados aquí.} .
\input{Rcmdr-menus-es}

La interfaz \textbf{R Commander} incluye algunos elementos añadidos a los menús y diálogos:

\begin{itemize}
\item Debajo de los menús hay una \textquotedblleft barra de herramientas\textquotedblright\ con una fila de botones.

\begin{itemize}
\item El botón (plano) más a la izquierda muestra el nombre del conjunto de datos activo. 
Inicialmente no hay ningún conjunto de datos activo. 
Si presiona el botón, podrá seleccionar entre los conjuntos de datos en memoria (si hay más de uno). 
La mayoría de los menús y diálogos de \textbf{R Commander} usan el conjunto de datos activo. 
(Los menús \emph{Fichero}, \emph{Edición} y \emph{Distribuciones} son excepciones.)

\item Dos botones permiten abrir el editor de datos de \textbf{R Commander} para modificar los datos del conjunto de datos activo o un visor para examinarlos\footnote{Por defecto, el editor de datos de  \textbf{R Commander} se usa si el número de valores (celdas) en el conjunto de datos es 10,000 o inferior; para grandes conjuntos de datos se usa el editor estándar de \textbf{R}. 
 Véase la ayuda de \textbf{R Commander} para cambiar este límite.}. 
El visor de datos puede permanecer abierto mientras se realizan otras operaciones y la vista se refresca si el conjunto de datos se modifica, por ejemplo añadiendo una variable\footnote{El visor de datos, proporcionado por la función \texttt{showData} de David Firth del paquete \textbf{relimp}~\citep{Firth16}, puede ser lento para conjuntos de datos con muchas variables. 
Cuando el número de variables excede a 100 o el número de casos excede de 20 000, el visor de la instrucción \texttt{View} de \textbf{R}, estéticamente menos agradable, se usa en su lugar para mostrar el conjunto de datos. 
Para usar \texttt{View} independientemente del número de variables o casos, ponga el límite en 0. 
Véase la ayuda de \textbf{R Commander} para más detalles.}.

\item Un botón plano indica el nombre del modelo estadístico activo --- un modelo lineal (como un modelo de regresión lineal), un modelo lineal generalizado, un modelo logit multinomial o un modelo de regresión ordinal o un modelo lineal generalizado de efectos fijos\footnote{Los paquetes con extensiones de \textbf{R Commander} (\citealp{Fox17}; \citealp{Fox07}; \citealp{FoxCarvalho12}) pueden proporcionar clases de modelos adicionales.}. 
Inicialmente no hay ningún modelo activo. 
Si hay en memoria más de un modelo asociado al conjunto de datos activo, puede elegir entre los modelos pulsando el botón. 
\textbf{R Commander} sincroniza los modelos y los conjuntos de datos con los que se corresponden.
\end{itemize}

\item Inmediatamente debajo de la barra de herramientas hay un panel que contiene la pestaña \emph{R Script}, una gran ventana de texto desplazable. 
Como se mencionó, las instrucciones generadas por \textbf{R Commander} se copian en esta ventana. 
Puede editar el texto en la pestaña \emph{Script} o incluso teclear sus propias instrucciones de \textbf{R} en la ventana.
Pulsando el botón \emph{Ejecutar}, que está en la parte derecha debajo de la pestaña \emph{R Script} (o, alternativamente, la combinación de teclas \emph{Ctrl-r}\footnote{Esto es, mantener pulsada la tecla \emph{Ctrl} (o \emph{Control}) y simultáneamente pulsar la tecla \emph{r}. En \textbf{macOS} usualmente se puede sustituir la tecla \emph{Instrucción} por la tecla \emph{Control}, pero tanto aquí como de manera general se pueden usar combinaciones con la tecla \emph{Control.}}, para \textquotedblleft
ejecutar,\textquotedblright\ o\emph{\ Ctrl-Tab}, causa que la línea que tiene el cursor sea enviada (o reenviada) para su ejecución. 
Si hay varias lineas seleccionadas (por ejemplo, pulsando el botón izquierdo y arrastrando el ratón sobre ellas), entonces presionando \emph{Ejecutar} todas ellas serán ejecutadas. 
Las instrucciones introducidas en la pestaña \emph{R Script} pueden ocupar más de una línea, pero todas las líneas deben enviarse simultáneamente. 
La combinación de teclas \emph{Ctrl-a} selecciona todo el texto en la pestaña \emph{R Script} y \emph{Ctrl-s} muestra una caja de diálogo para guardar el contenido de la pestaña.
La pestaña \emph{R Markdown} es descrita en la sección~\ref{sec-markdown}.

\item Debajo de las pestañas \emph{R Script} y \emph{R Markdown} hay un panel que contiene una gran ventana desplazable de texto para la \emph{Salida}.
Las instrucciones se muestran en el panel \emph{Salida} en rojo, los resultados en azul oscuro (como en la ventana estándar de \emph{R Console}).

\item En la parte inferior hay un pequeño panel gris para \emph{Mensajes}. 
Los mensajes de error se muestran en rojo, los avisos en verde y otros mensajes en azul oscuro.
Los errores y avisos también pueden ir acompañados de un aviso acústico.
\end{itemize}

Como se ha mencionado, una vez cargado el paquete \textbf{Rcmdr}, puede minimizar la consola \emph{R Console}. La ventana \textbf{R Commander} puede ser redimensionada o mazimizada de la forma usual. 
Si redimendiona \textbf{R Commander}, el ancho de las sucesivas salidas de \textbf{R} será automáticamente ajustado al panel \emph{Salida}.

\textbf{R Commander} es altamente configurable: Aquí se ha descrito la configuración estándar. 
Se pueden hacer cambios en la configuración usando el menú \emph{Herramientas}$\longrightarrow$\emph{\ Opciones\ldots}\ o, más extensamente, ajustando las opciones de \textbf{R Commander} en \textbf{R}\footnote{Una entrada de menú que termina en puntos suspensivos (i.e., tres puntos, ...) lleva a un cuadro de diálogo, este es el convenio para las interfaces gráficas. 
En este documento, $\longrightarrow$ representa seleccionar una entrada de menú o un submenú de un menú.}.
Véase \emph{Ayuda}$\longrightarrow$\emph{Ayuda de R Commander} para más detalles.


\section{Entrada de datos}

La mayoría de los procedimientos en \textbf{R Commander} asumen que hay un conjunto de datos activo\footnote{Los procedimientos seleccionados mediante el menú \emph{Distribuciones} son excepciones, así como \emph{Introducir y analizar una tabla de doble entrada...} en el menú \emph{Estadísticos}$\longrightarrow$\emph{Tablas de contingencia}.}. 
Si hay muchos conjuntos de datos en memoria, puede seleccionar entre todos ellos, pero sólo uno estará activo.
Cuando \textbf{R Commander} se inicia, no hay conjunto de datos activo.

\textbf{R Commander} proporciona muchos métodos para la introducción de datos en \textbf{R}:

\begin{itemize}
\item Usando el editor de datos de \textbf{R Commander}, puede introducir datos directamente vía \emph{Datos}$\longrightarrow$\emph{Nuevo conjunto de datos...}. 
Está es una elección razonable para un conjunto de datos muy pequeño.

\item Puede importar datos de un archivo de texto plano (\textquotedblleft ascii\textquotedblright) o del portapapeles, o desde Internet desde una URL, de otro paquete estadístico (\textbf{Minitab}, \textbf{SPSS}, \textbf{SAS}, o \textbf{Stata}) o desde una hoja de cálculo \textbf{Excel}.

\item Puede leer conjuntos de datos que estén incluidos en un paquete de \textbf{R}, tecleando el nombre del conjunto de datos (si lo conoce) o seleccionando el conjunto de datos en un cuadro de diálogo.
\end{itemize}

\subsection{Leyendo datos de un fichero de texto}

Por ejemplo, considere el fichero de datos \texttt{Nations.txt}\footnote{Este fichero está en el subdirectorio \texttt{etc} del paquete \textbf{Rcmdr}. 
Los datos son de 1998 de las Naciones Unidas.}. 
Las primeras líneas del fichero son como sigue:
\begin{verbatim}
TFR contraception infant.mortality GDP region
Afghanistan                 6.90    NA  154  2848   Asia
Albania                     2.60    NA   32   863   Europe
Algeria                     3.81    52   44  1531   Africa
American-Samoa              NA      NA   11  NA     Oceania
Andorra                     NA      NA  NA   NA     Europe
Angola                      6.69    NA  124   355   Africa
Antigua                     NA      53   24  6966   Americas
Argentina                   2.62    NA   22  8055   Americas
Armenia                     1.70    22   25   354   Europe
Australia                   1.89    76    6 20046   Oceania
. . .

\end{verbatim}

\begin{itemize}
\item La primera línea del fichero contiene los nombres de las variables: \texttt{TFR} (la tasa de fertilidad total\footnote{N.T.: Del inglés ``total fertility rate"}, expresada como número de niños por mujer), \texttt{contraception} (la tasa de anticonceptivos usados entre las mujeres casadas, en porcentaje), \texttt{infant.mortality} (tasa de mortalidad infantil por cada 1000 nacimientos vivos), \texttt{GDP} (renta percápita bruta\footnote{N.T.: Del inglés ``gross domestic product".}, en dólares estadounidenses) y \texttt{region}.

\item Las líneas siguiente contienen los datos, una línea por país.
Los valores de los datos están separados por \textquotedblleft espacios en blanco\textquotedblright\ --- uno o más espacios o tabuladores. 
Aunque es útil hacer que los datos se alinéen verticalmente, no es necesario hacerlo. 
Observe que las líneas de datos empiezan con el nombre del país. 
Porque queremos que éstos sean los \textquotedblleft nombres de las filas\textquotedblright\ para el conjunto de datos, no hay nombre de variable correspondiente:
Esto es, hay cinco variables pero seis valores en cada línea, el primero de los cuales es alfabético. 
Cuando esto ocurre la instrucción \texttt{read.table} de \textbf{R} interpreta el primer valor de cada línea como el nombre de la fila.

\item Algunos valores están perdidos. 
En \textbf{R}, es más conveniente usar \texttt{NA} (representando \textquotedblleft no disponible\textquotedblright\footnote{N.T.: NA del inglés ``Not available".}) para representar los valores perdidos, como hemos hecho aquí.

\item Las variables \texttt{TFR}, \texttt{contraception}, \texttt{infant.mortality} y \texttt{GDP} son variables numéricas (cuantitativas); en contraste, \texttt{region} contiene los nombres de las regiones. 
Cuando los datos son leídos, \textbf{R} tratará \texttt{region} como un \textquotedblleft factor\textquotedblright\ --- esto es, como una variable categórica. 
En la mayoría de los contextos, \textbf{R Commander} distingue entre variables numéricas y factores, e intentará evitar que haga cosas no razonables, como calcular la media de un factor\footnote{Las variables de caracteres, cuyos valores son cadenas de caracteres, y las variables lógicas, cuyos valores son \texttt{VERDADERO} o \texttt{FALSO} son tratadas como factores por \textbf{R Commander}.}.
\end{itemize}

Para leer el fichero \texttt{Nations.txt} en \textbf{R}, seleccione \emph{Datos}$\longrightarrow$\emph{Importar datos}$\longrightarrow$\emph{desde un fichero de texto, portapapeles o URL...} del menú de \emph{R Commander}. 
Esta operación mostrará un cuadro de diálogo \emph{Leer archivo de texto, portapapeles o URL}, como se muestra en la figura~\ref{fig-2}.
El nombre por defecto del conjunto de datos es \texttt{Dataset}. 
Hemos cambiado el nombre a \texttt{Nations}.

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=7.053500in,
%natwidth=7.546700in,
%height=3.6961in,
%width=3.9535in
scale=.7
]%
{fig-2-es}%
\caption{Leer archivo de texto, portapapeles o URL}%
\label{fig-2}%
\end{figure}

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=9.599800in,
%natwidth=13.159800in,
%height=5.0203in,
%width=6.8709in
scale=.35
]%
{fig-2b-es}%
\caption{Diálogo abrir fichero para leer datos de un fichero de texto.}%
\label{fig-2b}%
\end{figure}


Los nombre válidos en \textbf{R} comienzan con una letra mayúscula o minúscula (o un punto, \texttt{.}) y están compuestos íntegramente por letras, puntos, subrayados (\texttt{\_}), y dígitos (es decir, \texttt{0}--\texttt{9}); en particular no incluye blancos dentro del nombre del conjunto de datos. 
\textbf{R} es sensible a mayúsculas, así, por ejemplo, \texttt{nations}, \texttt{Nations} y \texttt{NATIONS} son distintos y podrían usarse para representar diferentes conjuntos de datos.

Pulsando el botón \emph{Aceptar} en el diálogo \emph{Leer archivo de texto, portapapeles o URL} se muestra el diálogo \emph{Abrir} fichero, mostrado en la figura~\ref{fig-2b}. 
Aquí se navega y se selecciona el fichero \texttt{Nations.txt}. 
Pulsando el botón \emph{Abrir} en el diálogo se lee el fichero de datos. 
Una vez leído el fichero de datos, se convierte en el conjunto de datos activo en \textbf{R Commander}. 
Consecuentemente, en la figura~\ref{fig-3}, el nombre del conjunto de datos aparece en el botón del conjunto de datos activo cerca de la parte superior izquierda de la ventana de \emph{R Commander}.

A continuación pulsamos el botón \emph{Visualiza conjunto de datos} para mostrar la ventana del visor de datos, como se muestra en la figura~\ref{fig-3}. 

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=7.946800in,
%natwidth=15.106600in,
%height=3.6048in,
%width=6.826in
scale=.35
]%
{fig-3-es}%
\caption{Mostrando el conjunto de datos activo.}%
\label{fig-3}%
\end{figure}

La instrucción \texttt{read.table} de \textbf{R} crea un \textquotedblleft data
frame,\textquotedblright\ que es un objeto que contiene un rectángulo de datos de casos por variables: Las filas del conjunto de datos representan los casos u observaciones y las columnas representan las variables. 
Los conjuntos de datos en \textbf{R Commander} son objetos data.frame de \textbf{R}.


\subsection{Introduciendo datos directamente}

Puede introducir datos directamente en el editor de datos tipo hoja de cálculo de \textbf{R Commander}. 
Una alternativa, que en realidad es preferible, es guardar los datos en un fichero de texto plano (tenga cuidado, si crea un archivo con un procesador de texto, de guardarlo como un fichero de texto plano o \textquotedblleft ascii\textquotedblright), típicamente un fichero de tipo \texttt{.txt} y entonces leer el fichero como en la sección precedente, vía \emph{Datos}$\longrightarrow$\emph{Importar datos}$\longrightarrow$\emph{de un archivo de texto, el portapapeles o URL\ldots}. 
Si sus datos ya están en un programa tipo hoja de cálculo, como \textbf{Excel}, simplemente puede exportar los datos a un fichero de texto de valores separados por comas (\texttt{.csv}) y leer el fichero en \textbf{R Commander}, teniendo cuidado de especificar como separador la coma. 
Recuerde que también puede leer una hoja de cálculo \textbf{Excel} directamente.

Como ejemplo de introducción directa de datos, usemos un conjunto de datos muy pequeño del problema~2.44 en~\citet{Moore00}:

\begin{itemize}
\item Seleccione \emph{Datos}$\longrightarrow$\emph{Nuevo conjunto de datos...} del menú de \emph{R Commander}. 
 Opcionalmente introduzca el nombre para el conjunto de datos, como \texttt{Problema2.44}, en el cuadro de diálogo resultante y pulse en el botón \emph{Aceptar}. 
(Recuerde que los nombres en \textbf{R} no pueden incluir blancos.) 
Esto mostrará la ventana del \emph{Editor de datos} con un conjunto de datos vacío.

\item Introduzca los datos del problema en las dos primeras columnas del editor. 
Añada una columna pulsando en el botón \emph{Añadir columna} de la barra del editor o seleccionando \emph{Añadir columna} desde el menú \emph{Editar}. 
Similarmente añada filas a los datos pulsando sobre el botón \emph{Añadir fila} repetidamente o vía el menú \emph{Editar}.
También puede aádir una file pulsando la telca \emph{Enter}  o añadir una columna pulsando la tecla \emph{Tab} cuando el cursor está en una celda de la tabla de datos.

\item Puede moverse de una celda a otra usando las teclas de cursor o pulsando el botón izquierdo del ratón en la celda a la que se quiere desplazar. 
Inicialmente, los nombres de las variables son \texttt{var1} y \texttt{var2}, y los valores de los datos son todos \texttt{NA} (es decir, omitidos). 
Cuando introduzca un nuevo nombre de variable, nombre de fila o un valor de un dato en una celda del editor de datos, el valor nuevo reemplaza al antiguo. 
Si hace una pulsación doble en una celda, entonces la celda pasa a valer \texttt{NA}. 
Cuando termine de introducir los datos, la ventana del editor de datos debería parecerse a la de la figura \ref{fig-4}. 

\item En este ejemplo, ambas variables son numéricas. 
Si introduce una variable no numérica en una columna del editor de datos (que no sea el valor \texttt{NA}), entonces la columna se considerará un factor (variable categórica) en el nuevo conjunto de datos. 

\begin{figure}[htb]%
\centering
\includegraphics[
height=2.5in,
]%
{fig-4-es}%
\caption{Editor de datos después de la edición}%
\label{fig-4}%
\end{figure}


\item Seleccione \emph{Archivo}$\longrightarrow$\emph{Salir y guardar} del menú \emph{Editor de datos} o pulse en el botón \emph{Aceptar}. 
El conjunto de datos introducido es ahora el conjunto de datos activo en \textbf{R Commander}.
\end{itemize}

\subsection{Leyendo datos desde paquetes\label{sec-data-in-packages}}

Muchos paquetes de \textbf{R} incluyen datos. 
Los conjuntos de datos en paquetes pueden ser listados en una ventana emergente vía \emph{Datos}$\longrightarrow$\emph{Datos en paquetes}$\longrightarrow$\emph{Lista de datos en paquetes} y pueden ser leídos en \textbf{R Commander} vía \emph{Datos}$\longrightarrow$\emph{Datos en paquetes}$\longrightarrow$\emph{Leer datos desde paquete adjunto}\footnote{No todos los datos en paquetes son conjuntos de datos (data.frame) pero sólo los conjuntos de datos son apropiados para su uso en \textbf{R Commander}. 
Si intenta leer datos que no sean un conjunto de datos (data.frame), \textbf{R Commander} intentará convertirlo en data.frae, mostrando un aviso si la conversión fue exitosa o un error si no.}. 
La caja de diálogo resultante se muestra en la figura~\ref{fig-6-5}
\footnote{El ejemplo muestra conjuntos de datos, incluido \texttt{Prestige}, que están en el paquete \textbf{car}; estos conjuntos de datos están actualmente en el paquete \textbf{carData}, que se carga junto con el paquete \textbf{car} cuando se inicia \textbf{R Commander}.}. 
Si se conoce el nombre del conjunto de datos en un paquete entonces se puede introducir directamente su nombre; en otro caso se hace una pulsación doble sobre el nombre del paquete para el que quiere mostrar los conjuntos de datos en la lista de la derecha y haciendo una pulsación doble en el nombre del conjunto de datos se copia el nombre en el campo de entrada del diálogo\footnote{En general en \textbf{R Commander}, cuando es necesario copiar un elemento de una lista a otra posición en el diálogo se realiza mediante una doble pulsación.}. 
Pulsando una letra en la lista \emph{Conjunto de datos} la lista se desplaza hasta el próximo conjunto de datos cuyo nombre empieza por esa letra. 
Puede acceder a paquetes adicionales de \textbf{R} que estén instalados en su biblioteca mediante \emph{Herramientas}$\longrightarrow$\emph{Cargar paquetes}.%


\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=3.813200in,
%natwidth=5.106700in,
%height=2.0108in,
width=2.6833in
]%
{fig-6-5-es}%
\caption{Leyendo datos desde paquete adjunto --- en este caso el conjunto de datos \texttt{Prestige} del paquete \textbf{car}.}%
\label{fig-6-5}%
\end{figure}


\section{Creando resúmenes numéricos y gráficos}

Una vez que hay un conjunto de datos activo, puede usar los menús de \textbf{R Commander} para obtener una gran variedad de resúmenes numéricos y gráficos. 
Vamos a describir simplemente algunos ejemplos básicos aquí. 
Una buena interfaz gráfica debe ser ampliamente autoexplicativa.
Si prefiere una introducción más extensa al \textbf{R Commander}, vea~\citet{Fox17}.

En el ejemplo a continuación, suponemos que el conjunto de datos activo es \texttt{Nations}, leído de un fichero de texto en la sección anterior. 
Si ha introducido el conjunto de datos de cinco observaciones de Moore (2000), o leído el conjunto de datos \texttt{Prestige} del paquete \textbf{car} --- las operaciones también descritas en la sección anterior --- entonces uno de esos es el conjunto de datos activo. 
Recuerde que puede cambiar el conjunto de datos activo pulsando sobre el botón con el nombre del conjunto de datos activo en la parte superior izquierda de la ventana de \emph{R Commander}, eligiendo entre la lista de conjuntos de datos disponibles en memoria.

Seleccionando \emph{Estadísticos}$\longrightarrow$\emph{Resúmenes}$\longrightarrow$\emph{Conjunto de datos activo} se obtiene el resultado de la figura~\ref{fig-7}. 
Para cada variable numérica en el conjunto de datos (\texttt{TFR}, \texttt{contraception}, \texttt{infant.mortality} y \texttt{GDP}), \textbf{R} muestra los valores mínimo y máximo, el primer y tercer cuartil, la mediana y la media, junto con el número de datos omitidos. 
Para la variable categórica \texttt{region}, se obtiene el número de observaciones en cada \textquotedblleft nivel\textquotedblright\ del factor. 
Cuando el conjunto de datos tiene más de diez variables, \textbf{R Commander} preguntará si realmente quiere proceder --- protegiéndonos de una indeseada y voluminosa salida ---. 
Este ítem de menú es inusual en el sentido de que directamente llama a una instrucción de \textbf{R} en vez de producir un cuadro de diálogo, como es más habitual en los menús de \textbf{R Commander}.

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=10.800300in,
%natwidth=9.293400in,
%height=5.6446in,
width=4.8609in
]%
{fig-7-es}%
\caption{Obtención de resúmenes de las variables para el conjunto de datos activo.}%
\label{fig-7}%
\end{figure}

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=4.466600in,
%natwidth=6.533000in,
%height=2.352in,
width=3.4255in
]%
{fig-8-es}%
\caption{La pestaña \emph{Datos} en el cuadro de diálogo \emph{Resúmenes numéricos}.}%
\label{fig-8}%
\end{figure}

Por ejemplo, seleccionando \emph{Estadísticos}$\longrightarrow$\emph{Resúmenes}$\longrightarrow$\emph{Resúmenes numéricos...} se obtiene el diálogo de la figura~\ref{fig-8}. 
Únicamente las variables numéricas aparecen en la lista del diálogo; el factor \texttt{region} se excluye debido a que no es apropiado realizar resúmenes numéricos como la media o la desviación típica de un factor. 
Seleccionamos la variable \texttt{infant.mortality} pulsando en ella\footnote{Para seleccionar una variable en una lista, simplemente pulse sobre su nombre. 
En otros contextos, cuando tenga (o quiera) seleccionar más de una variable, en esos casos, se aplican las normas generales en \textbf{Windows}: 
Pulsar sobre una variable la selecciona y deselecciona cualquier variable previamente seleccionada; pulsando en un nombre con la tecla mayúscula pulsada extiende la selección y pulsado con la tecla \emph{Ctrl} cambia el estado de selección para una variable.}. 
El diálogo \emph{Resúmenes numéricos} tiene dos pestañas: \emph{Datos} y \emph{Estadísticos}. 
Pulsando en \emph{Estadísticos} se selecciona la pestaña, como se muestra en la figura~\ref{fig-8a}. 
En este caso, tomaremos los estadísticos seleccionados por defecto. 
Pulsando en \emph{Aceptar}, se obtiene la siguiente salida (en el panel \emph{Salida}):
\begin{verbatim}
> numSummary(Nations[,"infant.mortality"], statistics=c("mean", "sd", "IQR",
+   "quantiles"), quantiles=c(0,.25,.5,.75,1))
     mean       sd IQR 0% 25% 50% 75% 100%   n NA
 43.47761 38.75604  54  2  12  30  66  169 201  6
\end{verbatim}%

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=4.466600in,
%natwidth=6.533000in,
%height=2.352in,
width=3.4255in
]%
{fig-8a-es}%
\caption{La pestaña \emph{Estadísticos} en el cuadro de diálogo \emph{Resúmenes numéricos}.}%
\label{fig-8a}%
\end{figure}

\noindent Por defecto, la instrucción de \textbf{R} que ejecutada muestra la media, la desviación típica (\texttt{sd}) y el rango intercuartílico\footnote{NT: interquartile range} (\texttt{IQR}) de la variable, junto con los cuantiles (percentiles) correspondientes al mínimo, el primer cuarto, la mediana, el tercer cuarto y el máximo, \texttt{n} es el número de observaciones válidas y \texttt{NA} el número de valores omitidos.

Como es habitual en los diálogos de \textbf{R Commander}, el diálogo \emph{Resúmenes numéricos} en la figura~\ref{fig-8} incluye los botones \emph{Ayuda}, \emph{Reiniciar}, \emph{Aceptar}, \emph{Cancelar} y \emph{Aplicar}\footnote{El orden de los botones cambia según el sistema operativo y es diferente, por ejemplo, en \textbf{Mac OS X} y en \textbf{Windows}.}. 
El botón \emph{Help} conduce a una página de ayuda (que aparece en el navegador) sobre el diálogo en sí mismo (como aquí) o sobre la función a la que llama el diálogo. 
El botón \emph{Reiniciar}, que está presente en la mayoría de los diálogos de \textbf{R Commander}, reinicia el diálogo a su estado original; en caso contrario, el diálogo mantiene la selección hecha en la última ocasión. 
El estado del diálogo también se reinicia cuando el conjunto de datos activo cambia. 
Como se mostró, el botón \emph{Aceptar} cierra el diálogo y genera una instrucción de \textbf{R}. 
El botón \emph{Aplicar} también genera una instrucción, pero reabre el diálogo en su último estado, facilitando la aplicación de varias operaciones similares. 
Si comete un error en el diálogo, --- por ejemplo, pulsando \emph{Aceptar} sin seleccionar una variable en el diálogo \emph{Resúmenes numéricos} --- un mensaje de error aparecerá y el diálogo se volverá a abrir. 


\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=6.400100in,
%natwidth=7.813200in,
%height=3.3574in,
width=4.0913in
]%
{fig-8-1-es}%
\caption{Seleccionado un variable grupo en el cuadro de diálogo de \emph{Grupos}.}%
\label{fig-8-1}%
\end{figure}

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=4.466600in,
%natwidth=6.533000in,
%height=2.352in,
width=3.4255in
]%
{fig-8-2-es}%
\caption{El cuadro \emph{Resúmenes numéricos} después de seleccionar la variable grupo \texttt{region} y dos variables numéricas.}%
\label{fig-8-2}%
\end{figure}

El diálogo \emph{Resúmenes numéricos} también proporciona los resúmenes en los grupos definidos por los niveles de un factor. 
Pulsando en el botón \emph{Resumir por grupos...} en la pestaña \emph{Datos} se obtiene el diálogo \emph{Grupos} como se muestra en la figura~\ref{fig-8-1}. 
Debido a que hay un único factor en el conjunto de datos \texttt{Nations}, sólo la variable \texttt{region} aparece en la lista y es preseleccionada; pulsando en \emph{Aceptar} el botón \emph{Resumir por grupos...} cambia a \emph{Resumir por region} (véase la figura~\ref{fig-8-2}). 
En este caso, hemos seleccionado dos variables numéricas para resumir, \texttt{GDP} e \texttt{infant.mortality}.
Pulsando \emph{Aceptar} se obtienen los resultados a continuación en el panel \emph{Salida}:
\begin{verbatim}
> numSummary(Nations[,c("GDP", "infant.mortality")], groups=Nations$region,
+   statistics=c("mean", "sd", "IQR", "quantiles"), quantiles=c(0,.25,.5,.75,1))

Variable: GDP
              mean        sd      IQR  0%     25%    50%      75%  100%  n NA
Africa    1196.000  2089.614   795.50  36  209.00  389.5  1004.50 11854 54  1
Americas  5398.000  6083.311  5268.50 386 1749.25 2765.5  7017.75 26037 40  1
Asia      4505.051  6277.738  6062.50 122  345.00 1079.0  6407.50 22898 39  2
Europe   13698.909 13165.412 24582.25 271 1643.75 9222.5 26226.00 42416 44  1
Oceania   8732.600 11328.708 16409.25 654 1102.75 2348.5 17512.00 41718 20  5

Variable: infant.mortality
             mean        sd  IQR 0%   25%  50%    75% 100%  n NA
Africa   85.27273 35.188095 50.0  7 61.00 85.0 111.00  169 55  0
Americas 25.60000 17.439713 24.0  6 12.00 21.5  36.00   82 40  1
Asia     45.65854 32.980001 50.0  5 22.00 37.0  72.00  154 41  0
Europe   11.85366  7.122363 10.0  5  6.00  8.0  16.00   32 41  4
Oceania  27.79167 29.622229 26.5  2  9.25 20.0  35.75  135 24  1
\end{verbatim}

\noindent Muchos otros diálogos de \textbf{R Commander} permiten seleccionar una variable grupo de esta forma. 

Hacer gráficos con \textbf{R Commander} también es directo. 
Por ejemplo, seleccionando \emph{Gráficos}$\longrightarrow$\emph{Histograma...} del menú de \textbf{R Commander} se obtiene el diálogo \emph{Histograma} en la figura~\ref{fig-9}. 
Están las pestañas \emph{Datos} y \emph{Opciones} en este diálogo. 
Tomamos las opciones por defecto (la pestaña \emph{Opciones} no se muestra) y pulsando en \texttt{infant.mortality} y a continuación en \emph{Aceptar}, se abre la ventana \emph{Graphics Device} con el histograma mostrado en la figura~\ref{fig-10}. 
Si hace varios gráficos en una sesión, únicamente el más reciente aparecerá en la ventana \emph{Graphics Device} window\footnote{En \textbf{Windows}, puede recuperar los gráficos anteriores usando la tecla \emph{Re Pág} y \emph{Av Pág} del teclado si previamente se ha activado la opción de histórico de gráficos en la ventana de gráficos, vía Histórico $\longrightarrow$ Grabación. 
En \textbf{macOS}, las flechas izquierda y derecha se pueden usar para moverse por los gráficos.
Los gráficos tridimensionales de puntos creados con \emph{Gráficos}$\longrightarrow$\emph{Gráfico 3D}$\longrightarrow$\emph{Nube de puntos 3D...} aparece en una ventana especial \emph{RGL device}.}.

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=3.933600in,
%natwidth=6.533000in,
%height=2.073in,
width=3.4255in
]%
{fig-9-es}%
\caption{El diálogo \emph{Histograma}. }%
\label{fig-9}%
\end{figure}

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=9.733500in,
%natwidth=9.173000in,
%height=4.4076in,
width=4.1569in
]%
{fig-10-es}%
\caption{Una ventana gráfica con el histograma para mortalidad infantil en el conjunto de datos \texttt{Nations}.}%
\label{fig-10}%
\end{figure}

\section{Modelos estadísticos}

Varios tipos de modelos estadísticos pueden ser ajustados en \textbf{R Commander} usando los opciones del menú \emph{Estadísticos}$\longrightarrow$\emph{Ajustar modelos}: modelos lineales (con \emph{Regresión lineal} y \emph{Modelo lineal}), modelos lineales generalizados, modelos logit multinomial y modelos de regresión ordinal como el modelo odds-proporcional [los dos últimos de los paquetes \textbf{nnet} y \textbf{MASS} de Venables y Ripley, respectivamente \citep{VenablesRipley02}], y modelos lineales y lineales generalizados [usando el paquete \textbf{lme4} de \citep{BatesMachlerBolker15}]. 
Aunque los diálogos resultantes tienen diferencias en algunos detalles (por ejemplo, el diálogo del modelo lineal generalizado permite seleccionar una familia de distribuciones y su correspondiente función de enlace), comparten una estructura general, como se ilustra en el diálogo \emph{Modelo lineal} de la figura~\ref{fig-10-5}\footnote{Una excepción es el diálogo \emph{Regresión lineal} en el que la variable respuesta y las variables explicativas son simplemente seleccionadas por el nombre de la lista conteniendo las variables numéricas en el conjunto de datos actual.}. 
Antes de seleccionar \emph{Estadísticos}$\longrightarrow$\emph{Ajustar modelos}$\longrightarrow$\emph{Modelo lineal}, hacemos \texttt{Prestige} el conjunto de datos activo pulsando en el botón conjunto de datos activo y seleccionando \texttt{Prestige} de la lista resultante. 
Recuerde que los datos \texttt{Prestige} se leyeron del paquete \textbf{car} en la sección~\ref{sec-data-in-packages}. 

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=5.667100in,
%natwidth=9.333300in,
%height=2.9747in,
width=4.8817in
]%
{fig-10-5-es}%
\caption{El diálogo \emph{Modelo lineal}, con \texttt{Prestige} del paquete \textbf{car} como conjunto de datos activo.}%
\label{fig-10-5}%
\end{figure}

\begin{itemize}
\item Una pulsación doble en una variable de la lista del diálogo la copia en la fórmula del modelo --- a la parte izquierda de la fórmula, si está vacía, o a la parte derecha en otro caso (con un signo + precediéndola si el contexto lo requiere). 
Los factores (variables categóricas --- aquí , \texttt{type}) son etiquetadas con paréntesis circundantes en la lista de variables\footnote{Algunos conjuntos de datos contienen variables lógicas (con valores \texttt{TRUE} y \texttt{FALSE}) y variables de carácter, con valores que son cadenas de texto (como es \texttt{"male"} y \texttt{"female"}). 
Si tales variables están presentes, \textbf{R Commander} las tratará como si fuesen factores. 
En la mayoría de los contextos, esto funciona bien. 
Los datos de tipo carácter leídos de un fichero de texto plano se convierten automáticamente en factores.}. 
Introduciendo un factor en el lado derecho del modelo estadístico se genera una variable auxiliar regresora. 

\item La fila superior de botones en la barra de herramientas debajo de la fórmula puede ser usada para introducir operadores y paréntesis en el lado derecho de la fórmula. 

\item La fila inferior de botones de la barra de herramientas puede ser usada para introducir términos splines o polinomiales en la fórmula del modelo, con los grados de libertad para los splines y polinomios controlados por la caja a la derecha de los botones (por defecto 5 grados de libertad y grado 2, respectivamente). 

\item También puede teclear directamente en el campo fórmula y puede que tenga que hacerlo, por ejemplo para poner un término como \texttt{log(income)} en la fórmula, como se ha hecho aquí. 
Alguna información sobre las fórmulas en los modelos de \textbf{R} puede ser obtenida pulsando en botón \emph{Ayuda fórmula del modelo} en el diálogo del modelo lineal. 

\item El nombre del modelo, aquí \texttt{LinearModel.1}, se genera automáticamente, pero puede sustituirlo por un nombre válido de \textbf{R}. 

\item Puede introducir una expresión de \textbf{R} en el campo etiquetado \emph{Expresión de selección}; si se suministra, se pasa al argumento \texttt{subset} de la función \texttt{lm} y es usado para ajustar el modelo a un subconjunto del conjunto de observaciones. 
Una forma de expresión de selección son las expresiones lógicas que se evalúan a \texttt{TRUE} o \texttt{FALSE} para cada observación, como \texttt{type != "prof"} (que seleccionará todas las ocupaciones no profesionales del conjunto de datos \texttt{Prestige}).

\item En la versión 2.8-0 de \textbf{R Commander} (o posterior), también hay un cuadro de texto en el que puede escribir los nombres o números de casos (filas) que se omitirán del modelo ajustado. (Este cuadro no se muestra en la figura~\ref{fig-10-5}).
  
\item Opcionalmente seleccionando una variable peso en el desplegable \emph{Pesos} se obtiene una regresión mínimo cuadrática ponderada. 
\end{itemize}

Pulsando el botón \emph{Aceptar} se genera la siguiente instrucción y la salida, y se establece \texttt{LinearModel.1} como el modelo activo, con su nombre mostrado en el botón \emph{Modelo}: 
\begin{verbatim}
> LinearModel.1 <- lm(prestige ~ (education + log(income))*type,
+   data=Prestige)

> summary(LinearModel.1)

Call:
lm(formula = prestige ~ (education + log(income)) * type, data = Prestige)

Residuals:
    Min      1Q  Median      3Q     Max
-13.970  -4.124   1.206   3.829  18.059

Coefficients:
                          Estimate Std. Error t value Pr(>|t|)
(Intercept)              -120.0459    20.1576  -5.955 5.07e-08 ***
education                   2.3357     0.9277   2.518  0.01360 *
log(income)                15.9825     2.6059   6.133 2.32e-08 ***
type[T.prof]               85.1601    31.1810   2.731  0.00761 **
type[T.wc]                 30.2412    37.9788   0.796  0.42800
education:type[T.prof]      0.6974     1.2895   0.541  0.58998
education:type[T.wc]        3.6400     1.7589   2.069  0.04140 *
log(income):type[T.prof]   -9.4288     3.7751  -2.498  0.01434 *
log(income):type[T.wc]     -8.1556     4.4029  -1.852  0.06730 .
---
Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

Residual standard error: 6.409 on 89 degrees of freedom
  (4 observations deleted due to missingness)
Multiple R-squared:  0.871, Adjusted R-squared:  0.8595
F-statistic: 75.15 on 8 and 89 DF,  p-value: < 2.2e-16
\end{verbatim}

Las operaciones con el modelo activo pueden ser seleccionadas el menú \emph{Modelos}. 
Por ejemplo, \emph{Modelos}$\longrightarrow$\emph{Tests de hipótesis}$\longrightarrow$\emph{Tabla ANOVA...}, seguido de la selección del test por defecto \textquotedblleft Tipo-II\textquotedblright, produce la siguiente salida: 
\begin{verbatim}
> Anova(LinearModel.1, type="II")
Anova Table (Type II tests)

Response: prestige
                 Sum Sq Df F value    Pr(>F)
education        1209.3  1 29.4446 4.912e-07 ***
log(income)      1690.8  1 41.1670 6.589e-09 ***
type              469.1  2  5.7103  0.004642 **
education:type    178.8  2  2.1762  0.119474
log(income):type  290.3  2  3.5344  0.033338 *
Residuals        3655.4 89
---
Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
\end{verbatim}

\section{Retazos}

\subsection{Produciendo informes\label{sec-markdown}}

En su configuración por defecto, \textbf{R Commander} incluye la pestaña \emph{R Markdown} en el panel superior, que acumula las instrucciones generadas durante la sesión en un documento \textbf{\ R Markdown}\footnote{\textbf{R Commander} puede también crear opcionalmente un documento \textbf{knitr} \textbf{LaTeX} \citep{Xie16} y compilarlo en un fichero PDF. 
Estas opciones requieren una instalación de \textbf{LaTeX} y son activadas poniendo la opción de \textbf{Rcmdr} \texttt{use.knitr} a \texttt{TRUE}. 
Véase, \emph{Ayuda}$\longrightarrow$\emph{Ayuda de R-Commander} y \emph{Herramientas}$\longrightarrow$\emph{Opciones}.}. 
La figura~\ref{fig-11} muestra la pestaña \emph{R Markdown} de la sesión actual, que ha sido desplazada hasta la parte superior del documento generado. 
Como su nombre indica, \textbf{R Markdown} es un lenguaje simple de marcado, que incluye bloques de instrucciones de \textbf{R} y puede ser usado para generar páginas HTML (web) u otro tipo de documento. 
Para más información sobre \textbf{R Markdown}, seleccione \emph{Ayuda}$\longrightarrow$\emph{Usando R Markdown} del menú de \textbf{R Commander}\footnote{Éste y otras entradas del menú \emph{Ayuda} pueden necesitar una conexión activa a internet.}.

\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=5.119900in,
%natwidth=9.439600in,
%height=2.6907in,
width=4.9373in
]%
{fig-11-es}%
\caption{La pestaña \emph{R Markdown}, con el botón \emph{Generar informe}.}%
\label{fig-11}%
\end{figure}


Cada instrucción de \textbf{R} generada por \textbf{R Commander} produce un bloque de intrucciones en el documento \textbf{R Markdown}\footnote{Las instrucciones que requieren interacción directa del usuario, como la identificación interactiva de puntos en un gráfico, son eliminadas del documento \textbf{R Markdown}. 
También, las instrucciones que generan errores son suprimidas del documento.}. 
Estos bloques son delimitados por \texttt{```\{r\}} al inicio y por \texttt{```} (tres tildes graves) al final del bloque. 
Incluso si no está familiarizado con las instrucciones de \textbf{R}, podrá ver la relación entre las instrucciones y la salida resultante en el panel \emph{Salida}.

La pestaña \emph{R Markdown} es editable y puede modificar y añadir texto en la pestaña. 
En general es seguro escribir el texto explicatorio que quiera \emph{entre} bloques de código de \textbf{R} (véase arriba). 
En general, no obstante, a menos que sepa que está haciendo, no debería modificar los bloques de código de \textbf{R} en el documento \textbf{R Markdown} o añadir sus propios bloques de código. 
No obstante, puede borrar bloques enteros de código, siempre que los bloques siguientes no dependan de ello. 
Querrá eliminar bloques de código que produzcan salidas no deseadas. 
Los bloques de instrucciones que producen errores son automáticamente eliminados. 
Puede eliminar el bloque más reciente seleccionando \emph{Borrar el último bloque de instrucciones Markdown} del menú \emph{Editar} o pulsando el botón derecho en la pestaña \emph{R Markdown} y seleccionando \emph{Borrar el último bloque Markdown} del menú contextual. 
El primer bloque de código que empieza \texttt{```\{r echo=FALSE\}}) establece algunas opciones del software del paquete \textbf{knitr} (Xie, 2013) que son usadas para procesar el texto \textbf{R Markdown} y este texto y este bloque no debería ser normalmente modificados.

Algunas líneas han sido omitidas (indicadas con \ldots), aquí está el documento \textbf{R Markdown} producido por la sesión actual:
\begin{verbatim}
---
title: "Replace with Main Title"
author: "Your Name"
date: "AUTOMATIC"
---


```{r echo=FALSE, message=FALSE}
# include this code chunk as-is to set options
knitr::opts_chunk$set(comment=NA, prompt=TRUE)
library(Rcmdr)
library(car)
library(RcmdrMisc)
```

```{r}
Nations <- read.table("C:/R/R-3.0.1patched/library/Rcmdr/etc/Nations.txt",
  header=TRUE, sep="", na.strings="NA", dec=".", strip.white=TRUE)
```

. . .


```{r}
data(Prestige, package="car")
```

. . .

Let us regress occupational prestige on the education and income levels of the occupations,
transforming income to linearize its relationship to prestige:

```{r}
LinearModel.1 <- lm(prestige ~ (education + log(income))*type,
  data=Prestige)
summary(LinearModel.1)
```

```{r}
Anova(LinearModel.1, type="II")
```

\end{verbatim}

\noindent Es probablemente innecesario explicar que normalmente debería reemplazar \textquotedblleft\texttt{Your\ Name}\textquotedblright\ con su nombre y reemplazar \textquotedblleft\texttt{Replace\ with\ Main\ Title} 
\textquotedblright\ con el título del informe que quiera crear. 
Tal vez, menos obviamente, puede introduccir texto explicativo arbitrario antes del primer bloque de instrucciones, \emph{entre} los bloques de código \textbf{R} --- esto es, entre la terminación de un bloque \texttt{```} y el incio del siguiente \texttt{```\{r\}} --- y después del último bloque de instrucciones. 
Puede aprovechar la ventaja del marcado simple proporcionado por \textbf{R Markdown}; por ejemplo, el texto encerrado entre asteriscos (por ejemplo, \texttt{*esto es importante*}) será puesto en \textit{itálica}. 
Para ilustrar, añadimos el texto \textquotedblleft\texttt{Let us regress occupational prestige ...} \textquotedblright\ inmediatamente antes del bloque de instrucciones de \textbf{R} que hace la regresión.

El documento \textbf{R Markdown} también incluye encabezados de sección generados automáticamente para algunos bloques de instrucciones de  \textbf{R}. 
Estos están precedidos por tres marcas de almohadilla (\texttt{\#\#\#}), que indican un encabezado de nivel 3. Cuando el \textbf{R Markdown} documentado se convierte en un informe (ver inmediatamente a continuación), los encabezados aparecen no solo en el texto sino también en un índice generado automáticamente. Puede editar estos encabezados para personalizarlos y también puede agregar sus propios encabezados, usando, por ejemplo, \texttt{\#} para un encabezado de nivel 1 y \texttt{\#\#} para un encabezado de nivel 2 .

Cuando acabe de editar el documento \textbf{R Markdown}, puede generar un informe a partir de él en forma de documento HTML (página web), documento \textbf{Word}, fichero de texto enriquecido o PDF pulsando el botón \emph{Generar informe} debajo de la pestaña \emph{R Markdown}\footnote{Para generar un documento \textbf{Word} o fichero de texto enriquecido debe instalar \textbf{Pandoc}; para generar fichero PDF, necesitará adicionalmente \LaTeX. 
Para instalar este software auxiliar adicional, use el menú \emph{Herramientas}.}.
Un informe HTML debe abrirse en su navegador y un informe PDF en un visor de PDF.
Los ficheros \textbf{Word} y los ficheros de texto enriquecido deben ser abiertos manualmente, pero pueden ser fácilmente editados después.
Los documentos \textbf{R Markdown} deben ser guardados usando el menú \emph{Fichero}.

También puede, más convenientemente, abrir un editor \textbf{R Markdown} en una ventana mayor y separada para editar el documento (véase figura~\ref{fig-12}): 
Seleccione \emph{Editor R Markdown} del menú \emph{Editar} de \textbf{R Commander}; pulse el botón derecho en la pestaña \emph{R Markdown} y seleccione \emph{Editar documento R Markdown} del menú contextual; o pulse la combinación de teclas \emph{Control-E} cuando el cursor esté en la pestaña \emph{R Markdown}. 
El editor soporta el menú usual \emph{Editar} y las instrucciones del menú contextual usando el botón derecho y también permite compilar el documento \emph{R Markdown} en un informe HTML. 
Pulsando en el botón \emph{Aceptar} en el editor se guardan sus cambios en la pestaña \emph{R Markdown} y pulsando \emph{Cancelar} descarta los cambios. 
Debajo del menú en el editor, hay una gran barra de herramientas autoexplicativa con varios botones; si posa el ratón sobre un botón se muestra una \textquotedblleft ayuda\textquotedblright.


Puede abrir la ventana del editor \textbf{R Markdown} al comienzo de una sesión y dejarla abierta mientras trabaja.
Las instrucciones generadas por el \textbf{R Commander} se introducirán tanto en la pestaña \emph{R Markdown} como en el editor, y podrá escribir texto en el editor sobre la marcha. Consulte la ayuda del editor para obtener más información.


\begin{figure}[htb]%
\centering
\includegraphics[
%natheight=10.306300in,
%natwidth=9.826400in,
%height=5.3873in,
width=5.1374in
]%
{fig-12-es}%
\caption{El editor de documentos \textbf{R Markdown}.}%
\label{fig-12}%
\end{figure}


\subsection{Guardando e imprimiendo la salida}

Puede guardar el texto de la salida directamente desde el menú \emph{Fichero} en \textbf{R Commander}; también puede guardar o imprimir un gráfico desde el menú \emph{Fichero} en la ventana \emph{Graphics Device} de \textbf{R}. 
Si prefiere no usar la pestaña \emph{R Markdown}, puede coger el texto de salida y los gráficos que quiera en un documento de un procesador de texto. 
De esta forma, puede intercalar la salida de \textbf{R} con sus notas y explicaciones. 
Este procedimiento, no obstante, tiene el inconveniente de que no es directamente reproducible, mientras que un documento \textbf{R Markdown} puede ser posteriormente ejecutado para reproducir el análisis, posiblemente con modificaciones. 

Abra un procesador de texto como \textbf{Word}, \textbf{OpenOffice Writer} o incluso \textbf{WordPad} de \textbf{Windows}. 
Para copiar texto del panel \emph{Salida}, marque el bloque de texto con el ratón, seleccione \emph{Copiar} del menú \emph{Editar} (o presione la combinación de teclas \emph{Ctrl-c} o el botón derecho en el panel y seleccione \emph{Copiar} del menú contextual), y pegue el texto en el documento mediante \emph{Editar}$\longrightarrow$\emph{Pegar} (o \emph{Ctrl-v } o botón derecho \emph{Pegar}), como haría en cualquier aplicación \textbf{Windows}.
Un punto a mencionar es que debería usar una fuente monoespacio (\textquotedblleft\texttt{tipográfica}\textquotedblright), como \emph{Courier New}, para la salida de texto \textbf{R}; de lo contrario el texto no se alineará correctamente.

Igualmente, para copiar un gráfico, seleccione \emph{Fichero}$\longrightarrow$\emph{Copiar al portapapeles}$\longrightarrow$\emph{como Metafichero} desde el menú \emph{Graphics Device} de \textbf{R}; entonces pegue el gráfico en el documento usando \emph{Editar}$\longrightarrow$\emph{Pegar} (o \emph{Ctrl-v} o botón derecho \emph{Pegar}). 
Alternativamente, puede usar \emph{Ctrl-w} para copiar el gráfico en \emph{Graphics Device} de \textbf{R} o pulsar el botón derecho en el gráfico para obtener el menú contextual, desde el que puede seleccionar \emph{Copiar como metafichero}\footnote{Como verá cuando examine estos menús, puede guardar el gráfico en una gran variedad de formatos en ficheros y en el portapapeles. 
El procedimiento sugerido aquí es directo, no obstante, generalmente genera gráficos de alta calidad.
Una vez más, esta descripción es aplicable a los sistemas \textbf{Windows}.}. 
Al final de su sesión de \textbf{R}, puede guardar o imprimir el documento que haya creado, proporcionando un registro con anotaciones de su trabajo.

Un método alternativo para guardar el texto y la salida gráfica es usar el menú \emph{Ficheros} de \textbf{R Commander} y \emph{Gráficos}$\longrightarrow$\emph{Guardar el gráfico a un fichero}.
Guardar la pestaña \emph{Script} mediante el menú \emph{Fichero}$\longrightarrow$\emph{Guardar instrucciones}, le permite reproducir su trabajo en futuras ocasiones.

\subsection{Introduciendo instrucciones en la pestaña Script}

La pestaña \emph{R Script} proporciona una forma sencilla de editar, introducir y ejecutat instrucciones.
Las instrucciones generadas por \textbf{R Commander} aparecen en la pestaña \emph{Script} y puede introducir y editar las instrucciones en la pestaña más o menos como en un editor. 
\textbf{R Commander} no proporciona una auténtica \textquotedblleft consola\textquotedblright\ de \textbf{R}, no obstante, y la pestaña \emph{Script} tiene algunas limitaciones. 
Por ejemplo, todas las líneas de una instrucción multilínea deben ser enviadas simultáneamente para ejecución.
Para programación seria con \textbf{R}, es preferible utilizar un editor de instrucciones como el proporcionado para \textbf{R} en las versiones de \textbf{Windows} y \textbf{maxOS} , o --- incluso mejor --- un editor de programas o un entorno integrado de desarrollo, como por ejemplo, \textbf{RStudio} \href{https://posit.co/}{posit.co}\footnote{\textbf{R Commander} corre bajo \textbf{RStudio}, en ese caso la salida y los mensajes de \textbf{R Commander} se dirigen por defecto a la consola de \textbf{R} en \textbf{RStudio}.}.

\subsection{Usando extensiones de R Commander\label{sec-plugins}}

Las extensiones de \textbf{R Commander} son paquetes que añaden capacidades a \textbf{R Commander}. 
Muchas de esas extensiones están actualmente disponibles en CRAN y pueden ser descargadas e instaladas de la manera habitual. 
Las extensiones normalmente añaden menús o entradas de menú y cajas de diálogos asociadas a \textbf{R Commander}. 
También pueden modificar o incluso suprimir menús existentes o diálogos. 
Una extensión de \textbf{R Commander} programada adecuadamente puede ser cargada directamente, en cuyo caso \textbf{R Commander} se carga con ella o desde \textbf{R Commander} usando \emph{Herramientas}$\longrightarrow$\emph{Cargar plugin(s) de Rcmdr}. 
En el último caso, \textbf{R Commander} se reiniciará para activar la extensión. 
Es posible utilizar muchas extensiones de forma simultánea, pero las extensiones pueden tener conflictos con otras.  
Por ejemplo, una extensión puede borrar un menú en el que otra intenta añadir un elemento. 

\subsection{Terminando la sesión de R}

Hay muchas formas de terminar su sesión. 
Por ejemplo, puede seleccionar del menú de \textbf{R Commander} \emph{Fichero}$\longrightarrow$\emph{Salir}$\longrightarrow$\emph{De Commander y R}. 
Se le solicitará confirmación y entonces si quiere guardar el contenido de las ventanas \emph{R Script}, \emph{Salida} y \emph{R Markdown}. 
También, puede seleccionar \emph{Fichero}$\longrightarrow$\emph{Salir} desde la consola \emph{R Console}; en este caso, se le preguntará si quiere guardar el espacio de trabajo de \textbf{R}, (por ejemplo, los datos que \textbf{R} mantiene en memoria); normalmente contestará \emph{No}.
Guardar el espacio de trabajo de \textbf{R} podría crear problemas en sucesivas sesiones.

%\bibliographystyle{apa}
\bibliography{Rcmdr-Manual}



\end{document}
